
! =================================================================================================================================
! MODULE       : livestock_parameters
!
! CONTACT      : orchidee-help _at_ listes.ipsl.fr
!
! LICENCE      : IPSL (2011)
! This software is governed by the CeCILL licence see ORCHIDEE/ORCHIDEE_CeCILL.LIC
!
!>\BRIEF        This module initializes all the livestock parameters in function of the
!!              number of animal types and of the values chosen by the user.
!!
!!\n DESCRIPTION:  This module allocates and initializes the livestock parameters in function of the number of animal
!!                 and the values of the parameters. \n
!!
!! REFERENCE(S)	: None
!!
!! SVN          :
!! $HeadURL: $
!! $Date: 2020-10-26 16:23:54 +0100 (lun. 26 oct. 2020) $
!! $Revision: 6911 $
!! \n
!_ ================================================================================================================================

MODULE livestock_parameters

  USE livestock_parameters_var
  USE constantes_lmtc
  USE constantes
  USE ioipsl
  USE ioipsl_para
  USE defprec

  IMPLICIT NONE

CONTAINS


!! ================================================================================================================================
!! SUBROUTINE   : livestock_parameters_main
!!
!>\BRIEF          This subroutine initializes all the livestock parameters in function of the
!! number of vegetation types chosen by the user.
!!
!! DESCRIPTION  : This subroutine is called after the reading of the number of livestock classes and the options
!!                activated by the user in the configuration files. \n
!!                The allocation is done just before reading the correspondence table  between PFTs and MTCs
!!                defined by the user in the configuration file.\n
!!                With the correspondence table, the subroutine can initialize the pft parameters in function
!!                of the flags activated (ok_sechiba, ok_stomate, routing,...) in order to
!!                optimize the memory allocation. \n
!!                If the number of PFTs and pft_to_mtc are not found, the standard configuration will be used
!!                (13 PFTs, PFT = MTC). \n
!!                Some restrictions : the pft 1 can only be the bare soil and it is unique. \n
!!                Algorithm : Build new PFT from 13 generic-PFT or meta-classes.
!!                1. Read the number of PFTs in "run.def". If nothing is found, it is assumed that the user intend to use
!!                   the standard of PFTs (13).
!!                2. Read the index vector in "run.def". The index vector associates one PFT to one meta-classe (or generic PFT).
!!                   When the association is done, the PFT defined by the user inherited the default values from the meta classe.
!!                   If nothing is found, it is assumed to use the standard index vector (PFT = MTC).
!!                3. Check consistency
!!                4. Memory allocation and initialization.
!!                5. The parameters are read in the configuration file in config_initialize (control module).
!!
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S): None
!!
!! REFERENCE(S) : None
!!
!! FLOWCHART    : None
!! \n
!_ ================================================================================================================================

  SUBROUTINE livestock_parameters_main()

    IMPLICIT NONE

    !! 0. Variables and parameters declaration

    !! 0.4 Local variables

    INTEGER(i_std) :: j                             !! Index (unitless)

    !_ ================================================================================================================================

    !
    ! PFT global
    !

    IF(l_first_livestock_parameters) THEN

       !! 1. First time step
       IF(printlev>=3) THEN
          WRITE(numout,*) 'l_first_pft_parameters :we read the parameters from the def files'
       ENDIF

       !! 2. Memory allocation for the pfts-parameters
       CALL livestock_parameters_alloc()

       !! 3. Correspondance table

       !! 3.1 Initialisation of the correspondance table
       !! Initialisation of the correspondance table
       IF (nlm == nlmc) THEN
          livestock_to_lmtc = (/ 1, 2, 3, 4, 5 /)
       ELSE
          livestock_to_lmtc(:) = undef_int
       ENDIF !(nlm  == nlmc)

       !! 3.2 Reading of the conrrespondance table in the .def file
       !
       !Config Key   = LIVESTOCK_TO_LMTC
       !Config Desc  = correspondance array linking a livestock to NLMC
       !Config if    = OK_SECHIBA or OK_STOMATE
       !Config Def   = 1, 2, 3, 4, 5
       !Config Help  =
       !Config Units = [-]
       CALL getin_p('LIVESTOCK_TO_LMTC',livestock_to_lmtc)

       !! 3.3 If the user want to use the standard configuration, he needn't to fill the correspondance array
       !!     If the configuration is wrong, send a error message to the user.
       IF(nlm /= nlmc ) THEN
          !
          IF(livestock_to_lmtc(1) == undef_int) THEN
             STOP ' The array LIVESTOCK_TO_LMTC is empty : we stop'
          ENDIF !(livestock_to_lmtc(1) == undef_int)
          !
       ENDIF !(nlm /= nlmc )

       !! 3.4 Some error messages

       !! 3.4.1 What happened if livestock_to_lmtc(j) > nlmc or livestock_to_lmtc(j) <=0 (if the lmtc doesn't exist)?
       DO j = 1, nlm ! Loop over # PFTs
          !
          IF( (livestock_to_lmtc(j) > nlmc) .OR. (livestock_to_lmtc(j) <= 0) ) THEN
             WRITE(numout,*) 'the metaclass chosen does not exist'
             STOP 'we stop reading pft_to_mtc'
          ENDIF !( (livestock_to_lmtc(j) > nlmc) .OR. (livestock_to_lmtc(j) <= 0) )
          !
       ENDDO  ! Loop over # livestock


       !! 4.Initialisation of the pfts-parameters
       CALL livestock_parameters_init()

       !! 5. Useful data

       !! 5.1 Read the name of the livestocks given by the user
       !
       !Config Key   = LIVESTOCK_NAME
       !Config Desc  = Name of a livestock
       !Config if    = OK_SECHIBA or OK_STOMATE
       !Config Help  = the user can name the new PFTs he/she introducing for new species
       !Config Units = [-]
       CALL getin_p('LIVESTOCK_NAME',livestock_name)

       !! 5.2 A useful message to the user: correspondance between the number of the livestock
       !! and the name of the associated lmtc
       IF (printlev >=1 ) THEN
          WRITE(numout,*) ''
          DO j = 1,nlm ! Loop over # PFTs
             WRITE(numout,*) 'The livestock',j, 'called ', trim(LIVESTOCK_name(j)),' corresponds to the LMTC : ',trim(LMTC_name(livestock_to_lmtc(j)))
          END DO
          WRITE(numout,*) ''
       END IF


       !! 6. End message
       IF (printlev>=3) WRITE(numout,*) 'livestock_parameters_done'

       !! 8. Reset flag
       l_first_livestock_parameters = .FALSE.

    ELSE

       RETURN

    ENDIF !(l_first_livestock_parameters)

  END SUBROUTINE livestock_parameters_main




!! ================================================================================================================================
!! SUBROUTINE   : livestock_parameters_clear
!!
!>BRIEF         This subroutine deallocates memory at the end of the simulation.
!!
!! DESCRIPTION  : None
!!
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S): None
!!
!! REFERENCE(S) : None
!!
!! FLOWCHART    : None
!!

!_ ================================================================================================================================

  SUBROUTINE livestock_parameters_clear

    l_first_livestock_parameters = .TRUE.
    IF (ALLOCATED(LIVESTOCK_name)) DEALLOCATE(LIVESTOCK_name)
    IF (ALLOCATED(is_ruminant)) DEALLOCATE(is_ruminant)
    IF (ALLOCATED(weight_euref)) DEALLOCATE(weight_euref)
    IF (ALLOCATED(xexcreted)) DEALLOCATE(xexcreted)
    IF (ALLOCATED(xurine)) DEALLOCATE(xurine)
    IF (ALLOCATED(straw_euref)) DEALLOCATE(straw_euref)
    IF (ALLOCATED(n_straw_euref)) DEALLOCATE(n_straw_euref)
    IF (ALLOCATED(length_euref)) DEALLOCATE(length_euref)
    IF (ALLOCATED(xgrazing)) DEALLOCATE(xgrazing)
    IF (ALLOCATED(xyard)) DEALLOCATE(xyard)
    IF (ALLOCATED(graz_period)) DEALLOCATE(graz_period)
    IF (ALLOCATED(xmin)) DEALLOCATE(xmin)
    IF (ALLOCATED(ximm)) DEALLOCATE(ximm)
    IF (ALLOCATED(ef_hou)) DEALLOCATE(ef_hou)
    IF (ALLOCATED(ef_hou_s)) DEALLOCATE(ef_hou_s)
    IF (ALLOCATED(ef_hou_l)) DEALLOCATE(ef_hou_l)
    IF (ALLOCATED(ef_yard)) DEALLOCATE(ef_yard)
    IF (ALLOCATED(ef_sto)) DEALLOCATE(ef_sto)
    IF (ALLOCATED(ef_sto_l)) DEALLOCATE(ef_sto_l)
    IF (ALLOCATED(ef_sto_s)) DEALLOCATE(ef_sto_s)
    IF (ALLOCATED(ef_n2o_sto_l)) DEALLOCATE(ef_n2o_sto_l)
    IF (ALLOCATED(ef_n2o_sto_s)) DEALLOCATE(ef_n2o_sto_s)
    IF (ALLOCATED(ef_no_sto_l)) DEALLOCATE(ef_no_sto_l)
    IF (ALLOCATED(ef_no_sto_s)) DEALLOCATE(ef_no_sto_s)
    IF (ALLOCATED(ef_n2_sto_l)) DEALLOCATE(ef_n2_sto_l)
    IF (ALLOCATED(ef_n2_sto_s)) DEALLOCATE(ef_n2_sto_s)
    IF (ALLOCATED(ef_app_l)) DEALLOCATE(ef_app_l)
    IF (ALLOCATED(ef_app_s)) DEALLOCATE(ef_app_s)
    IF (ALLOCATED(ef_graz)) DEALLOCATE(ef_graz)
END SUBROUTINE

  SUBROUTINE livestock_parameters_alloc()

    IMPLICIT NONE

    !! 0. Variables and parameters declaration

    !! 0.1 Input variables

    !! 0.4 Local variables

    LOGICAL :: l_error                             !! Diagnostic boolean for error allocation (true/false)
    INTEGER :: ier                                 !! Return value for memory allocation (0-N, unitless)

    !_ ================================================================================================================================


    !
    ! 1. Parameters used anytime
    !

    l_error = .FALSE.

    ALLOCATE(LIVESTOCK_name(nlm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for LIVESTOCK_name. We stop. We need nlm words = ',nlm
       STOP 'livestock_parameters_alloc'
    END IF

    ALLOCATE(is_ruminant(nlm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for ruminant flag. We stop. We need nlm words = ',nlm
       STOP 'livestock_parameters_alloc'
    END IF

  !
  ! ANIMAL CHARACTERISTICS
  !

    ALLOCATE(weight_euref(nlm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for weight_euref. We stop. We need nlm words = ',nlm
       STOP 'livestock_parameters_alloc'
    END IF

    ALLOCATE(xexcreted(nlm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for xexcreted. We stop. We need nlm words = ',nlm
       STOP 'livestock_parameters_alloc'
    END IF

    ALLOCATE(xurine(nlm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for xurine. We stop. We need nlm words = ',nlm
       STOP 'livestock_parameters_alloc'
    END IF
  !
  ! ACTIVITIES CHARACTERISTICS
  !

    ALLOCATE(straw_euref(nlm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for straw_euref. We stop. We need nlm words = ',nlm
       STOP 'livestock_parameters_alloc'
    END IF

    ALLOCATE(n_straw_euref(nlm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for n_straw_euref. We stop. We need nlm words = ',nlm
       STOP 'livestock_parameters_alloc'
    END IF

    ALLOCATE(length_euref(nlm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for length_euref. We stop. We need nlm words = ',nlm
       STOP 'livestock_parameters_alloc'
    END IF

    ALLOCATE(xgrazing(nlm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for xgrazing. We stop. We need nlm words = ',nlm
       STOP 'livestock_parameters_alloc'
    END IF

    ALLOCATE(xyard(nlm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for xyard. We stop. We need nlm words = ',nlm
       STOP 'livestock_parameters_alloc'
    END IF

    ALLOCATE(graz_period(nlm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for graz_period. We stop. We need nlm words = ',nlm
       STOP 'livestock_parameters_alloc'
    END IF


  !
  ! MANURE INFORMATION
  !

    ALLOCATE(xmin(nlm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for xmin. We stop. We need nlm words = ',nlm
       STOP 'livestock_parameters_alloc'
    END IF

    ALLOCATE(ximm(nlm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for ximm. We stop. We need nlm words = ',nlm
       STOP 'livestock_parameters_alloc'
    END IF

    ALLOCATE(ef_hou(nlm,nfrac_man),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for ef_hou. We stop. We need nlm words = ',nlm,2
       STOP 'livestock_parameters_alloc'
    END IF

    ALLOCATE(ef_hou_s(nlm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for ef_hou_s. We stop. We need nlm words = ',nlm
       STOP 'livestock_parameters_alloc'
    END IF



    ALLOCATE(ef_hou_l(nlm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for ef_hou_l. We stop. We need nlm words = ',nlm
       STOP 'livestock_parameters_alloc'
    END IF

    ALLOCATE(ef_yard(nlm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for ef_yard. We stop. We need nlm words = ',nlm
       STOP 'livestock_parameters_alloc'
    END IF

    ALLOCATE(ef_sto(nlm,nspecies,nfrac_man),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for ef_sto. We stop. We need nlm words = ',nlm*4*2
       STOP 'livestock_parameters_alloc'
    END IF

    ALLOCATE(ef_sto_s(nlm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for ef_sto_s. We stop. We need nlm words = ',nlm
       STOP 'livestock_parameters_alloc'
    END IF

    ALLOCATE(ef_sto_l(nlm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for ef_sto_l. We stop. We need nlm words = ',nlm
       STOP 'livestock_parameters_alloc'
    END IF

    ALLOCATE(ef_n2o_sto_l(nlm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for ef_n2o_sto_l. We stop. We need nlm words = ',nlm
       STOP 'livestock_parameters_alloc'
    END IF

    ALLOCATE(ef_n2o_sto_s(nlm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for ef_n2o_sto_s. We stop. We need nlm words = ',nlm
       STOP 'livestock_parameters_alloc'
    END IF

    ALLOCATE(ef_no_sto_l(nlm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for ef_no_sto_l. We stop. We need nlm words = ',nlm
       STOP 'livestock_parameters_alloc'
    END IF

    ALLOCATE(ef_no_sto_s(nlm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for ef_no_sto_s. We stop. We need nlm words = ',nlm
       STOP 'livestock_parameters_alloc'
    END IF

    ALLOCATE(ef_n2_sto_l(nlm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for ef_n2_sto_l. We stop. We need nlm words = ',nlm
       STOP 'livestock_parameters_alloc'
    END IF

    ALLOCATE(ef_n2_sto_s(nlm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for ef_n2_sto_s. We stop. We need nlm words = ',nlm
       STOP 'livestock_parameters_alloc'
    END IF

    ALLOCATE(ef_app_l(nlm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for ef_app_l. We stop. We need nlm words = ',nlm
       STOP 'livestock_parameters_alloc'
    END IF

    ALLOCATE(ef_app_s(nlm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for ef_app_s. We stop. We need nlm words = ',nlm
       STOP 'livestock_parameters_alloc'
    END IF

    ALLOCATE(ef_graz(nlm),stat=ier)
    l_error = l_error .OR. (ier /= 0)
    IF (l_error) THEN
       WRITE(numout,*) ' Memory allocation error for ef_graz. We stop. We need nlm words = ',nlm
       STOP 'livestock_parameters_alloc'
    END IF
END SUBROUTINE livestock_parameters_alloc

!! ================================================================================================================================
!! SUBROUTINE   : livestock_parameters_init
!!
!>BRIEF          This subroutine initializes all the livestock parameters by the default values
!! of the corresponding metaclasse.
!!
!! DESCRIPTION  : This subroutine is called after the reading of the number of LIVESTOCK types and the correspondence
!!                table defined by the user in the configuration files.

!!                With the correspondence table, the subroutine can search the default values for the parameter
!!                even if the LIVESTOCK types are classified in a random order.

!!
!! RECENT CHANGE(S):
!!
!! MAIN OUTPUT VARIABLE(S): None
!!
!! REFERENCE(S) : None
!!
!! FLOWCHART    : None
!!

!_ ================================================================================================================================

  SUBROUTINE livestock_parameters_init()

    IMPLICIT NONE

    !! 0. Variables and parameters declaration

    !! 0.1 Input variables

    !! 0.4 Local variables

    INTEGER(i_std)                :: jv            !! Index (unitless)
    !_ ============================================================================================================================

    !
    ! 1. Correspondance between the LIVESTOCK type values and thes LMTCs values
    !
  LIVESTOCK_name(:) = LMTC_name(livestock_to_lmtc(:))
  is_ruminant(:) = is_ruminant_lmtc(livestock_to_lmtc(:))
  !
  ! Max aboveground biomass fraction that can be extracted
  !
  max_above = max_above_lmtc
  !
  ! ANIMAL CHARACTERISTICS
  !
  weight_euref(:) = weight_euref_lmtc(livestock_to_lmtc(:))
  xexcreted(:) = xexcreted_lmtc(livestock_to_lmtc(:))
  xurine(:) = xurine_lmtc(livestock_to_lmtc(:))
  !
  ! ACTIVITIES CHARACTERISTICS
  !
  straw_euref(:) = straw_euref_lmtc(livestock_to_lmtc(:))
  n_straw_euref(:) = n_straw_euref_lmtc(livestock_to_lmtc(:))
  length_euref(:) = length_euref_lmtc(livestock_to_lmtc(:))
  xgrazing(:) = xgrazing_lmtc(livestock_to_lmtc(:))
  xyard(:) = xyard_lmtc(livestock_to_lmtc(:))
  !
  ! MANURE INFORMATION
  !
  xmin(:) = xmin_lmtc(livestock_to_lmtc(:))
  ximm(:) = ximm_lmtc(livestock_to_lmtc(:)) 
  ef_hou_s(:) = ef_hou_s_lmtc(livestock_to_lmtc(:))
  ef_hou_l(:) = ef_hou_l_lmtc(livestock_to_lmtc(:))
  ef_yard(:) = ef_yard_lmtc(livestock_to_lmtc(:))
  ef_sto_l(:) = ef_sto_l_lmtc(livestock_to_lmtc(:))
  ef_sto_s(:) = ef_sto_s_lmtc(livestock_to_lmtc(:))
  ef_n2o_sto_l(:) = ef_n2o_sto_l_lmtc(livestock_to_lmtc(:))
  ef_n2o_sto_s(:) = ef_n2o_sto_s_lmtc(livestock_to_lmtc(:))
  ef_no_sto_l(:) = ef_no_sto_l_lmtc(livestock_to_lmtc(:))
  ef_no_sto_s(:) = ef_no_sto_s_lmtc(livestock_to_lmtc(:))
  ef_n2_sto_l(:) = ef_n2_sto_l_lmtc(livestock_to_lmtc(:))
  ef_n2_sto_s(:) = ef_n2_sto_s_lmtc(livestock_to_lmtc(:))
  ef_app_l(:) = ef_app_l_lmtc(livestock_to_lmtc(:))
  ef_app_s(:) = ef_app_s_lmtc(livestock_to_lmtc(:))
  ef_graz(:) = ef_graz_lmtc(livestock_to_lmtc(:))

END SUBROUTINE livestock_parameters_init

!! ================================================================================================================================
!! SUBROUTINE   : config_livestock_parameters
!!
!>BRIEF          This subroutine will read the imposed values for the global livestock
!! parameters
!!
!! DESCRIPTION  : None
!!
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S): None
!!
!! REFERENCE(S) : None
!!
!! FLOWCHART    : None
!!

!_ ================================================================================================================================

  SUBROUTINE config_livestock_parameters

    IMPLICIT NONE

    !! 0. Variables and parameters declaration

    !! 0.4 Local variable

    INTEGER(i_std) :: jv                   !! Index (untiless)

    !_ ============================================================================================================================

		!Config Key   = LIVESTOCK_NAME
		!Config Desc =  description of the MTC (unitless)
		!Config if =
		!Config Def =(/'Nondairycattle','Dairycattle','Pig','Smallruminants','Chicken'/)!5
		!Config Help =
		!Config Units = [unitless]
		CALL getin_p('LIVESTOCK_NAME',LIVESTOCK_name)

    !Config Key   = MAX_ABOVE
    !Config Desc =  Max aboveground biomass fraction that can be extracted (unitless)
    !Config if =
    !Config Def =(/0.7/)
    !Config Help =
    !Config Units = [unitless]
    CALL getin_p('MAX_ABOVE',max_above)


    !Config Key   = IS_RUMINANT
    !Config Desc  = flag for ruminant types
    !Config if    = OK_SECHIBA or OK_STOMATE
    !Config Def   = y, y, n, y, n
    !Config Help  =
    !Config Units = [BOOLEAN]
    CALL getin_p('IS_RUMINANT',is_ruminant)


		!Config Key   = WEIGHT_EUREF_LMTC
		!Config Desc =  European weight references  (kg)
		!Config if =
		!Config Def =(/340.0,600.0,100.0,50.0,1.6/)
		!Config Help = used in case of normalizing litter amount
		!Config Units = [kg]
		CALL getin_p('WEIGHT_EUREF',weight_euref)

		!Config Key   = XEXCRETED
		!Config Desc =  proportion of N excreted calculated from (1-Nretained)
		!Config if =
		!Config Def =(/0.93,0.80,0.70,0.90,0.70/)
		!Config Help = Nretained : fraction of nitrogen retained by the animal for the production of meat, milk, or wood
		!Config Units = [1-Nretained]
		CALL getin_p('XEXCRETED',xexcreted)

		!Config Key   = XURINE
		!Config Desc =  Proportion of TAN in N excreted
		!Config if =
		!Config Def =(/0.6,0.6,0.7,0.5,0.7/)
		!Config Help =
		!Config Units = [1-Nretained]
		CALL getin_p('XURINE',xurine)

		!Config Key   = STRAW_EUREF
		!Config Desc =  European straw used in litter-based manure
		!Config if =
		!Config Def =(/500.0,1500.0,400.0,20.0,undef/)
		!Config Help = management systems  (kg /Heads/yr) used only for Europe region?
		!Config Units = [1-Nretained]
		CALL getin_p('STRAW_EUREF',straw_euref)

		!Config Key   = N_STRAW_EUREF
		!Config Desc =  European N added in straw-litter
		!Config if =
		!Config Def =(/2.0,6.0,1.6,0.08,undef/)
		!Config Help = (kg N /Heads/yr)
		!Config Units = [1-Nretained]
		CALL getin_p('N_STRAW_EUREF',n_straw_euref)

		!Config Key   = LENGTH_EUREF
		!Config Desc =  European length of housing (days)
		!Config if =
		!Config Def =(/180,180,365,30,365/)
		!Config Help =
		!Config Units = [days]
		CALL getin_p('LENGTH_EUREF',length_euref)

		!Config Key   = XGRAZING
		!Config Desc =  Fraction of the year where
		!Config if =
		!Config Def =(/0.5,0.5,0.0,0.92,0.0/)
		!Config Help = animal graze (European)
		!Config Units = [days]
		CALL getin_p('XGRAZING',xgrazing)

		!Config Key   = XYARD
		!Config Desc =  Fraction of the year where
		!Config if =
		!Config Def =(/0.1,0.25,undef,0.02,undef/)
		!Config Help = animal is at yard (European)
		!Config Units = [days]
		CALL getin_p('XYARD',xyard)

		!Config Key   = XMIN
		!Config Desc =  Fraction of mineralized organic N
		!Config if =
		!Config Def =(/0.1,0.1,0.1,0.1,0.1/)
		!Config Help = present in manure stored
		!Config Units = [days]
		CALL getin_p('XMIN',xmin)

		!Config Key   = XIMM
		!Config Desc =  Fraction of TAN that is immobilized in
		!Config if =
		!Config Def =(/0.0067,0.0067,0.0067,0.0067,0.0067/)
		!Config Help = organic matter when manure is handled as straw-based solid manure
		!Config Units = [days]
		CALL getin_p('XIMM',ximm)

         	!Config Key   = EF_HOU_S
		!Config Desc =  Emission factor of TAN during
		!Config if =
		!Config Def =(/0.08,0.08,0.23,0.22,0.21/)
		!Config Help = housing in solid manure
		!Config Units = [days]
		CALL getin_p('EF_HOU_S',ef_hou_s)

		!Config Key   = EF_HOU_L
		!Config Desc =  Emission factor of TAN during
		!Config if =
		!Config Def =(/0.19,0.19,0.27,undef,undef/)
		!Config Help = housing in liquid manure
		!Config Units = [days]
		CALL getin_p('EF_HOU_L',ef_hou_l)


		!Config Key   = EF_YARD
		!Config Desc =  Emission factor of TAN when
		!Config if =
		!Config Def =(/0.53,0.30,undef,0.75,undef/)
		!Config Help = animal is at yard
		!Config Units = [days]
		CALL getin_p('EF_YARD',ef_yard)

		!Config Key   = EF_STO_L
		!Config Desc =  Emission factor of TAN during
		!Config if =
		!Config Def =(/0.25,0.25,0.11,undef,undef/)
		!Config Help = storage in liquid manure
		!Config Units = [days]
		CALL getin_p('EF_STO_L',ef_sto_l)

		!Config Key   = EF_STO_S
		!Config Desc =  Emission factor of TAN during
		!Config if =
		!Config Def =(/0.32,0.32,0.29,0.30,0.19/)
		!Config Help = storage in solid manure
		!Config Units = [days]
		CALL getin_p('EF_STO_S',ef_sto_s)

		!Config Key   = EF_N2O_STO_L
		!Config Desc =  Emission factor of N2O during
		!Config if =
		!Config Def =(/0.0,0.0,0.0,0.0,0.0/)
		!Config Help = storage in liquid manure
		!Config Units = [days]
		CALL getin_p('EF_N2O_STO_L',ef_n2o_sto_l)

		!Config Key   = EF_N2O_STO_S
		!Config Desc =  Emission factor of N2O during
		!Config if =
		!Config Def =(/0.02,0.02,0.01,0.02,0.02/)
		!Config Help = storage in solid manure
		!Config Units = [days]
		CALL getin_p('EF_N2O_STO_S',ef_n2o_sto_s)

		!Config Key   = EF_NO_STO_L
		!Config Desc =  Emission factor of NO during
		!Config if =
		!Config Def =(/0.0001,0.0001,0.0001,undef,undef/)
		!Config Help = storage in liquid manure
		!Config Units = [days]
		CALL getin_p('EF_NO_STO_L',ef_no_sto_l)

		!Config Key   = EF_NO_STO_S
		!Config Desc =  Emission factor of NO during
		!Config if =
		!Config Def =(/0.01,0.01,0.01,0.01,0.1/)
		!Config Help = storage in solid manure
		!Config Units = [days]
		CALL getin_p('EF_NO_STO_S',ef_no_sto_s)

		!Config Key   = EF_N2_STO_L
		!Config Desc =  Emission factor of N2 during
		!Config if =
		!Config Def =(/0.003,0.003,0.003,undef,undef/)
		!Config Help = storage in liquid manure
		!Config Units = [days]
		CALL getin_p('EF_N2_STO_L',ef_n2_sto_l)

		!Config Key   = EF_N2_STO_S
		!Config Desc =  Emission factor of N2 during
		!Config if =
		!Config Def =(/0.30,0.30,0.30,0.30,0.30/)
		!Config Help = storage in solid manure
		!Config Units = [days]
		CALL getin_p('EF_N2_STO_S',ef_n2_sto_s)

		!Config Key   = EF_APP_L
		!Config Desc =  Emission factor of TAN during
		!Config if =
		!Config Def =(/0.27,0.27,0.20,undef,undef/)
		!Config Help = application of liquid manure
		!Config Units = [days]
		CALL getin_p('EF_APP_L',ef_app_l)

		!Config Key   = EF_APP_S
		!Config Desc =  Emission factor of TAN during
		!Config if =
		!Config Def =(/0.68,0.68,0.45,0.90,0.41/)
		!Config Help = application of solid manure
		!Config Units = [days]
		CALL getin_p('EF_APP_S',ef_app_s)

		!Config Key   = EF_GRAZ
		!Config Desc =  Emission factor of TAN during
		!Config if =
		!Config Def =(/0.14,0.14,0.31,0.09,undef/)
		!Config Help = grazing
		!Config Units = [days]
		CALL getin_p('EF_GRAZ',ef_graz)

  END SUBROUTINE config_livestock_parameters

END MODULE livestock_parameters
