! =================================================================================================================================
! MODULE       : emis_nh3
!
! CONTACT      : orchidee-help _at_ listes.ipsl.fr
!
! LICENCE      : IPSL (2006)
! This software is governed by the CeCILL licence see ORCHIDEE/ORCHIDEE_CeCILL.LIC
!
!>\BRIEF       Calculate ammonia emissions from livestock
!!
!!\n DESCRIPTION: None
!!
!! RECENT CHANGE(S): None
!!
!! SVN		:
!! $HeadURL: svn://forge.ipsl.jussieu.fr/orchidee/perso/maureen.beaudor/ORCHIDEE_N_MANURE/ORCHIDEE/src_stomate/stomate_soilcarbon.f90 $
!! $Date: 2020-05-06 23:59:02 +0200 (mer. 06 mai 2020) $
!! $Revision: 6782 $
!! \n
!_ ================================================================================================================================

MODULE emis_nh3

  ! modules used:

  USE ioipsl_para
  USE mod_orchidee_para
  USE stomate_data
  USE constantes
  USE constantes_soil
  USE xios_orchidee
  USE livestock_parameters
  USE livestock_parameters_var
  USE grid

  IMPLICIT NONE

  ! private & public routines

  PRIVATE
  PUBLIC grazing_intensity, cn_conversion, biomass_th_estim, biomass_eff_estim, &
   diag_distr , manure_emis ! à modifier

CONTAINS



    ! ================================================================================================================================
    !!  SUBROUTINE   : cn_conversion
    !!
    !>\BRIEF        Computes the CN values used to convert carbon into nitrogen masses
    !!
    !!  CN used in the theoretical and effective biomass calculations
    !!
    !! MAIN OUTPUTS VARIABLE(S): cn_grass, cn_crop
    !! REFERENCE(S)   :
    !_ ================================================================================================================================

      SUBROUTINE cn_conversion (npts, harvest_above_lastyear, graz_above_lastyear,&
        cn_crop, cn_grass)

        !! 0. Variable and parameter declaration

          !! 0.1 Input variables
    INTEGER(i_std), INTENT(in)                            :: npts             !! Domain size (unitless)
    REAL(r_std), DIMENSION(npts,nelements), INTENT(in)    :: harvest_above_lastyear    !! Yearly harvested above ground biomass
                                                                                    !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nelements), INTENT(in)    :: graz_above_lastyear   !! Yearly grazed above ground biomass by ruminants
                                                                              !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex

    REAL(r_std), INTENT(inout)          :: cn_crop            ! CN of the crop biomass
    REAL(r_std), DIMENSION(npts), INTENT(inout)         :: cn_grass            ! CN of the grass biomass

    !! 0.2 Local variables

    INTEGER(i_std)                                :: ik      
    REAL(r_std),DIMENSION(nelements)               ::  net_harvest_above_yearly_tot ! Global yearly harvested above ground biomass
    REAL(r_std),DIMENSION(nelements)               :: net_harvest_above_yearly_sum ! yearly local harvest above biomass
    !_ ================================================================================================================================
    IF (printlev>=2) WRITE(numout,*) 'CN_conversion'

    ! CN calculation for crop and grass biomass
    ! CN crop is global avec cont_frac
   DO ik=1,nelements 
     net_harvest_above_yearly_sum(ik) = &
            &  SUM(harvest_above_lastyear(:,ik)*area(:)*contfrac(:))
     CALL allreduce_sum(net_harvest_above_yearly_sum(ik),net_harvest_above_yearly_tot(ik))
  END DO  

   IF (net_harvest_above_yearly_tot(initrogen) .GT. min_stomate) THEN
      cn_crop = net_harvest_above_yearly_tot(icarbon) / net_harvest_above_yearly_tot(initrogen)
    ELSE
      cn_crop = 0.
    END IF
    WHERE (graz_above_lastyear(:,initrogen) .GT. min_stomate)
      cn_grass(:) = graz_above_lastyear(:,icarbon) / graz_above_lastyear(:,initrogen)
    ELSEWHERE
      cn_grass(:) = 0.
    END WHERE

    IF (printlev>=2) WRITE( numout,*)'CN crop',cn_crop


    END SUBROUTINE


  ! ================================================================================================================================
  !!  SUBROUTINE   : biomass_th_estim
  !!
  !>\BRIEF        Computes the theoretical estimations of the biomass used for feed
  !!              and as litter
  !!  - Compute biomass of grass and crop that will be be eaten by each animal
  !!  (unit : kg C/m2/yr) + straw litter
  !! - Compute biomass of crop that will be be used as litter by each animal
  !!  (unit : kg C/m2/yr)
  !!  - Dry matter intake is : 2-3% (0.025) of the weight of the live animal
  !!  - C mass = 0.45 * dry matter
  !!
  !!
  !! MAIN OUTPUTS VARIABLE(S): sum_biomass_th_crop_tot, frac_grass, frac_crop_tot,
  !! frac_lit, intake
  !!
  !! REFERENCE(S)   :
  !!     - distr : gridded livestock density for 2006 from Robinson et al., 2014
  !!  (regridded at the ORCHIDEE's resolution)
  !!  - diet : regional grass and crop proportion in diet (adaptated from GLEAM 2011)
  !!  - weight : regional animal weights (adaptated from GLEAM 2011)
  !_ ================================================================================================================================

    SUBROUTINE biomass_th_estim (npts,distr, weight, diet, intake, sum_biomass_th_grass, sum_biomass_th_crop_tot, &
             frac_grass, frac_crop_tot, frac_lit, biomass_th_grass, biomass_th_crop_alim, biomass_th_crop_lit)

  !! 0. Variable and parameter declaration

      !! 0.1 Input variables

      INTEGER(i_std), INTENT(in)                            :: npts             !! Domain size (unitless)

      REAL(r_std), DIMENSION(npts,nlm), INTENT(in)          :: distr           ! Livestock distribution map (Heads/km2/yr) per points
      REAL(r_std), DIMENSION(npts,nlm), INTENT(in)          :: weight          ! Livestock weight map (kg) per points
      REAL(r_std), DIMENSION(npts,nlm), INTENT(in)          :: diet            ! Livestock diet map (-) per points

      !! 0.2 Output variables
      !! 0.3 Modified variables
      REAL(r_std), DIMENSION(npts,nlm), INTENT(inout)       :: intake           !! Intake required for each animal (kg C/year/ani)
      REAL(r_std), DIMENSION(npts), INTENT(inout)          :: sum_biomass_th_grass !! Sum of the estimated grass biomass ingested by all the animals
                                                                                  !! kg C/m2/yr
      REAL(r_std), DIMENSION(npts), INTENT(inout)          :: sum_biomass_th_crop_tot !! Sum of the estimated crop biomass ingested by all the animals
                                                                                  !! kg C/m2/yr
      REAL(r_std),DIMENSION(npts,nlm), INTENT(out)        :: frac_grass         !! Proportion of importance of each animal in grass biomass
      REAL(r_std),DIMENSION(npts,nlm), INTENT(out)        :: frac_crop_tot      !! Proportion of importance of each animal in crop biomass
      REAL(r_std),DIMENSION(npts,nlm), INTENT(out)        :: frac_lit           !! Proportion of importance of each animal in crop biomass used as litter
      REAL(r_std), DIMENSION(npts,nlm), INTENT(inout)        :: biomass_th_grass  !! Carbon biomass from grass estimated for each animal
                                                                                !! kg C/m2/yr
      REAL(r_std), DIMENSION(npts,nlm), INTENT(inout)        :: biomass_th_crop_alim !! Feed crop biomass required for each animal (kg C/m2/yr)
      REAL(r_std), DIMENSION(npts,nlm), INTENT(inout)        :: biomass_th_crop_lit !! Theoretical crop biomass used as litter by each animal (kg C/m2/yr)
      !! 0.4 Local variables

      REAL(r_std), DIMENSION(npts,nlm)                      :: biomass_th      !! Carbon biomass estimated for each animal
                                                                                !! kg C/m2/yr
      REAL(r_std), DIMENSION(npts,nlm)                      :: biomass_th_crop_tot !! Total crop biomass required for each animal (kg C/m2/yr)
      INTEGER(i_std)                                        :: nlmi    !! Indices (unitless)

     REAL(r_std)                                      ::  biomass_th_crop_yearly_sum
     REAL(r_std)                                      ::  biomass_th_crop_yearly_tot
     REAL(r_std)                                      ::  biomass_th_grass_yearly_sum
     REAL(r_std)                                      ::  biomass_th_grass_yearly_tot
     REAL(r_std)                                      ::  biomass_th_lit_yearly_sum
     REAL(r_std)                                      ::  biomass_th_lit_yearly_tot

  !_ ================================================================================================================================


  !! printlev is the level of diagnostic information, 0 (none) to 4 (full)
  IF (printlev>=2) WRITE(numout,*) 'Entering biomass_estim'


      !! 1. Get livestock input maps : distribution and weight
      DO nlmi=1, nlm ! loop over livestock category
        intake(:,nlmi) = 0.45 * 0.025 * weight(:,nlmi) * 365             ! kg DM/day/ani => kg C/year/ani
        biomass_th(:,nlmi) = distr(:,nlmi) * intake(:,nlmi) * 1e-6      ! kg C/km2/yr => kg C/m2/yr
        biomass_th_crop_alim(:,nlmi) = biomass_th(:,nlmi) * (1-diet(:,nlmi)) ! split the biomass into crop and grass
        biomass_th_grass(:,nlmi) = biomass_th(:,nlmi) * diet(:,nlmi)
        ! If we considere a straw C/N of 80 (according to usda) => C = 0.32*FM
        biomass_th_crop_lit(:,nlmi) = 0.32 * distr(:,nlmi) * straw_euref(nlmi) * 1e-6   ! kg FM /km2/yr => kg C/m2/yr
        biomass_th_crop_tot(:,nlmi) = biomass_th_crop_alim(:,nlmi) + biomass_th_crop_lit(:,nlmi)
        ! perhaps we should make some maps of straw used as well ?
        ! because there is no litter used in many regions
      END DO

      ! 2. Sum over animal dim to get the total biomass used as grass and crop
      sum_biomass_th_grass(:) = SUM(biomass_th_grass(:,:),dim=2)
      sum_biomass_th_crop_tot(:) = SUM(biomass_th_crop_tot(:,:),dim=2)

      ! 3. Proportion of importance of each animal in the resource extraction :
      DO nlmi=1, nlm ! loop over livestock category
        WHERE(sum_biomass_th_grass(:) .GT. min_stomate)
        frac_grass(:,nlmi) = biomass_th_grass(:,nlmi) / sum_biomass_th_grass(:)
       ELSEWHERE
        frac_grass(:,nlmi) = 0.
       END WHERE 
       WHERE (sum_biomass_th_crop_tot(:) .GT. min_stomate)
        frac_crop_tot(:,nlmi) = biomass_th_crop_tot(:,nlmi) / sum_biomass_th_crop_tot(:)
        ELSEWHERE
        frac_crop_tot(:,nlmi) = 0.
       END WHERE
     ! 4 Total of crop needs for litter use or feeding
       WHERE (biomass_th_crop_tot(:,nlmi) .GT. min_stomate)
        frac_lit(:,nlmi) = biomass_th_crop_lit(:,nlmi)/biomass_th_crop_tot(:,nlmi)
       ELSE WHERE
        frac_lit(:,nlmi) = 0.
       END WHERE
      END DO

      biomass_th_lit_yearly_sum = &
         & SUM(SUM(biomass_th_crop_lit(:,:), dim=2)*area*contfrac(:))
      CALL allreduce_sum(biomass_th_lit_yearly_sum,biomass_th_lit_yearly_tot)

      biomass_th_crop_yearly_sum = &
        &  SUM(sum_biomass_th_crop_tot(:)*area(:)*contfrac(:))
      CALL allreduce_sum(biomass_th_crop_yearly_sum,biomass_th_crop_yearly_tot)
     
      biomass_th_grass_yearly_sum = &
        &  SUM(sum_biomass_th_grass(:)*area(:)*contfrac(:))
      CALL allreduce_sum(biomass_th_grass_yearly_sum,biomass_th_grass_yearly_tot)

      IF (printlev>=2) WRITE( numout,*)'biomass_th_lit_yearly_tot in Pg C:',biomass_th_lit_yearly_tot*1e-12 

      IF (printlev>=2) WRITE( numout,*)'biomass_th_crop_yearly_tot in Pg C:',biomass_th_crop_yearly_tot*1e-12
      IF (printlev>=2) WRITE( numout,*)'biomass_th_grass_yearly_tot in Pg C:',biomass_th_grass_yearly_tot*1e-12

      IF (printlev>=4) WRITE(numout,*) 'Leaving biomass_th_estim'

      END SUBROUTINE

      ! ================================================================================================================================
      !!  SUBROUTINE   : grazing_intensity
      !!
      !>\BRIEF        Computes the estimation of the actual grass biomass available for animal
      !!              Constrain the resource use with the local available grass
      !!
      !!  Subroutine called independantly in StomateLpj
      !!
      !! MAIN OUTPUTS VARIABLE(S): gi
      !! REFERENCE(S)   :
      !_ ================================================================================================================================

        SUBROUTINE grazing_intensity (npts,veget_cov_max, gi, bm_alloc_lastyear, sum_biomass_th_grass)


  !! 0. Variable and parameter declaration

      !! 0.1 Input variables

      INTEGER(i_std), INTENT(in)                            :: npts             !! Domain size (unitless)
      REAL(r_std), DIMENSION(npts,nvm), INTENT(in)          :: veget_cov_max       !! new "maximal" coverage fraction of a PFT (LAI ->
                                                                                !! infinity) on ground @tex $(m^2 m^{-2})$ @endtex
      REAL(r_std), DIMENSION(npts), INTENT(in)          :: sum_biomass_th_grass !! Sum of the estimated grass biomass ingested by all the animals
                                                                                !! kg C/m2/yr
      REAL(r_std), DIMENSION(npts,nvm,nparts,nelements), INTENT(in)     :: bm_alloc_lastyear !! Last year's biomass increase, i.e. NPP per plant part
                                                                                !! gC/m2/day
      !! 0.2 Output variables

      !! 0.3 Modified variables

      REAL(r_std), DIMENSION(npts), INTENT(inout)          :: gi                !! Grazing Intensity
      
                                                                                               
      !! 0.4 Local variables

      INTEGER(i_std)                                        :: j, ij    !! Indices (unitless)
      ! Estimations of the biomasses ingested
      REAL(r_std), DIMENSION(npts,nvm)                      :: bm_grass_above   !! Yearly carbon biomass increase, i.e. NPP per plant part
      REAL(r_std), DIMENSION(npts)                          :: bm_grassland_above !! Yearly carbon biomass increase, i.e. NPP per plant part
                                                                                  !! normalised over grassland pfts

  !_ ================================================================================================================================
    IF (printlev>=2) WRITE(numout,*) 'grazing intensity'

    ! 1. Calculation of the grass biomass available in grassland PFTs

    bm_grassland_above(:) = zero
    DO j = 2,nvm
      IF ( .NOT. is_tree(j) .AND. natural(j) ) THEN
         bm_grass_above(:,j) = (bm_alloc_lastyear(:,j,ileaf,icarbon) + &
                             bm_alloc_lastyear(:,j,isapabove,icarbon) + &
                             bm_alloc_lastyear(:,j,ifruit,icarbon))* 1e-3 ! gC/m2/year into kgC/m2/year
             bm_grassland_above(:) = bm_grassland_above(:)+bm_grass_above(:,j) * veget_cov_max(:,j)
           END IF
         END DO
     
    WHERE(bm_grassland_above(:) .GT. min_stomate)
    ! 2. Calculation of the grazing intensity from above ground productivity
    gi(:) = MIN(max_above, sum_biomass_th_grass(:)/bm_grassland_above(:))
    ELSEWHERE
    gi(:) = 0.
    END WHERE

    !IF (printlev>=2) WRITE (numout,*) 'gi(:):',gi(:)
    !IF (printlev>=2) WRITE (numout,*) 'sum_biomass_th_grass', sum_biomass_th_grass(:)
    !IF (printlev>=2) WRITE (numout,*) 'bm_grass_above pft1', bm_grass_above(:,1)
    !IF (printlev>=2) WRITE (numout,*) 'bm_grassland_above', bm_grassland_above(:)
    END SUBROUTINE



    ! ================================================================================================================================
    !!  SUBROUTINE   : biomass_eff_estim
    !!
    !>\BRIEF
    !!        - Computes the estimation of the actual biomass available for animal
    !!        - Different adjustements are done : according to diet composition
    !!          and the global crop production
    !!        - Conversion from C into N is done with the variables harvest_above
    !!          and graz_above that are already given in carbon and in nitrogen
    !!
    !! Subroutine called in manure_emis to compute emissions from nitrogen masses used
    !!
    !! MAIN OUTPUTS VARIABLE(S):
    !! REFERENCE(S)   :
    !_ ================================================================================================================================

      SUBROUTINE biomass_eff_estim (npts, diet, &
        graz_above_lastyear,harvest_above_lastyear, &
        sum_biomass_th_grass,biomass_th_crop_alim, &
        frac_grass,frac_crop_tot,frac_lit,&
        biomass_eff_grass,biomass_eff_crop_alim,biomass_eff_crop_lit)


  !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    INTEGER(i_std), INTENT(in)                            :: npts             !! Domain size (unitless)
    REAL(r_std), DIMENSION(npts,nlm), INTENT(in)          :: diet              ! Livestock diet map (-) per points
    REAL(r_std), DIMENSION(npts,nelements), INTENT(in)    :: harvest_above_lastyear    !! Yearly harvested above ground biomass
                                                                              !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nelements), INTENT(in)     :: graz_above_lastyear   !! Yearly grazed above ground biomass by ruminants
                                                                                    !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex
    !REAL(r_std), DIMENSION(npts), INTENT(in)               :: area            !! Area
    !REAL(r_std), DIMENSION(npts), INTENT(in)               :: contfrac        !!  Fraction of continent in the grid cell (unitless)
    REAL(r_std), DIMENSION(npts), INTENT(in)               :: sum_biomass_th_grass !! Sum of the estimated grass biomass ingested by all the animals
    REAL(r_std), DIMENSION(npts,nlm), INTENT(in)        :: biomass_th_crop_alim !! Theoretical crop biomass ingested by each animal (kg C/m2/yr)    
    REAL(r_std),DIMENSION(npts,nlm),INTENT(in)            :: frac_lit         !! Proportion of importance of each animal in crop biomass used as litter
    REAL(r_std),DIMENSION(npts,nlm),INTENT(in)            :: frac_grass         !! Proportion of importance of each animal in grass biomass
    REAL(r_std),DIMENSION(npts,nlm),INTENT(in)           :: frac_crop_tot         !! Proportion of importance of each animal in crop biomass

    !! 0.2 Output variables
    !! 0.3 Modified variables
    REAL(r_std),DIMENSION(npts,nlm),INTENT(inout)      :: biomass_eff_grass    !! Actual grass biomass used by each animal (kg C/m2/yr)
    REAL(r_std),DIMENSION(npts,nlm),INTENT(inout)      :: biomass_eff_crop_lit  !! Actual crop biomass used as litter by each animal (kg C/m2/yr)
    REAL(r_std),DIMENSION(npts,nlm),INTENT(inout)      :: biomass_eff_crop_alim ! Actual crop biomass ingested by each animal (kg C/m2/yr)

    !! 0.4 Local variables

    INTEGER(i_std)                                        :: nlmi,j,ij    !! Indices (unitless)
    REAL(r_std),DIMENSION(npts,nlm)                       :: frac_crop_tot_eff !! Proportion of importance of each animal in crop biomass (effective for test)
    REAL(r_std), DIMENSION(npts)                          :: sum_biomass_th_crop_tot !! Sum of the estimated crop biomass ingested by all the animals
                                                                                !! kg C/m2/yr
    REAL(r_std),DIMENSION(npts)                           :: sum_biomass_eff_grass ! Sum of the effective grass biomass ingested by all the animals
                                                                                !! kg C/m2/yr
    REAL(r_std),DIMENSION(npts)                           :: sum_biomass_eff_crop_tot ! Sum of the effective crop biomass used by all the animals
                                                                                  !! kg C/m2/yr

    REAL(r_std),DIMENSION(npts,nlm)                       :: biomass_eff_crop_tot  ! Actual crop biomass used by each animal (kg C/m2/yr)
    REAL(r_std)                                           :: hi_eff       !! Proportion of harvested crop used for livestock feed / litter
    REAL(r_std)                                           :: net_harvest_above_yearly_tot ! Global yearly harvested above ground biomass
    REAL(r_std)                                           :: net_harvest_above_yearly_sum ! yearly local harvest above biomass
    REAL(r_std)                                           :: net_graz_above_yearly_tot ! Global yearly harvested above ground biomass
    REAL(r_std)                                           :: net_graz_above_yearly_sum ! yearly local harvest above biomass
    REAL(r_std)                                           :: biomass_eff_crop_tot_yearly_sum ! yearly local harvest above biomass for livestock
    REAL(r_std)                                           :: biomass_eff_crop_tot_yearly_tot ! Global yearly harvest above biomass for livestock
   REAL(r_std)                                           ::  biomass_eff_grass_yearly_sum ! yearly local harvest above biomass for livestock
   REAL(r_std)                                           :: biomass_eff_grass_yearly_tot ! Global yearly harvest above biomass for livestock
  !_ =============================================================================
  !! printlev is the level of diagnostic information, 0 (none) to 4 (full)
   IF (printlev>=2) WRITE(numout,*) 'Entering biomass_eff_estim'

    sum_biomass_eff_grass(:) = sum_biomass_th_grass(:)

    ! 2- Global yearly harvest above biomass :
    ! Parallel processing, gather the information from different processors. first argument is the
    ! local variable, the second argument is the global variable. bcast send it to all processors.
    net_harvest_above_yearly_sum = &
        &  SUM(harvest_above_lastyear(:,icarbon)*area(:)*contfrac(:))*1e-3     !gC/m2 gridcell into kgC/m2 gridcell
    CALL allreduce_sum(net_harvest_above_yearly_sum,net_harvest_above_yearly_tot)

    net_graz_above_yearly_sum = &
       & SUM(graz_above_lastyear(:,icarbon)*area*contfrac(:))*1e-3
    CALL allreduce_sum(net_graz_above_yearly_sum, net_graz_above_yearly_tot)

    IF (printlev>=2) WRITE (numout,*) 'net_harvest_above_yearly_tot in Pg C:',net_harvest_above_yearly_tot*1e-12
    IF (printlev>=2) WRITE (numout,*) 'net_graz_above_yearly_tot in Pg C:', net_graz_above_yearly_tot*1e-12

    ! 3- Adjustement with the biomass grazed locally over one year
    sum_biomass_eff_grass(:) = MIN(graz_above_lastyear(:,icarbon)* 1e-3,sum_biomass_eff_grass(:)) ! ! gC/m2/year into kgC/m2/year
    
        DO nlmi=1, nlm ! loop over livestock category
          biomass_eff_grass(:,nlmi) = sum_biomass_eff_grass(:)*frac_grass(:,nlmi)
          ! 4- Crop adjustement according to diet compostition
          WHERE(diet(:,nlmi) .GT. min_stomate)
             biomass_eff_crop_alim(:,nlmi) = biomass_eff_grass(:,nlmi) *((1-diet(:,nlmi))/diet(:,nlmi))
          ELSE WHERE
             biomass_eff_crop_alim(:,nlmi) = biomass_th_crop_alim(:,nlmi)
          END WHERE
          WHERE(frac_lit(:,nlmi) .LT. (1-min_stomate))
             biomass_eff_crop_lit(:,nlmi) = biomass_eff_crop_alim(:,nlmi) *(frac_lit(:,nlmi)/(1-frac_lit(:,nlmi)))
          END WHERE
          biomass_eff_crop_tot(:,nlmi) = biomass_eff_crop_alim(:,nlmi) + biomass_eff_crop_lit(:,nlmi)
       END DO 
 
      biomass_eff_crop_tot_yearly_sum = &
   & SUM(SUM(biomass_eff_crop_alim(:,:), dim=2)*area(:)*contfrac(:))

   CALL allreduce_sum(biomass_eff_crop_tot_yearly_sum,biomass_eff_crop_tot_yearly_tot)
   IF (printlev>=2) WRITE (numout,*)'biomass_eff_crop_alim_yearly_tot in PgC',biomass_eff_crop_tot_yearly_tot*1e-12
   
   ! Global litter 
   biomass_eff_crop_tot_yearly_sum = &
   & SUM(SUM(biomass_eff_crop_lit(:,:), dim=2)*area(:)*contfrac(:))

   CALL allreduce_sum(biomass_eff_crop_tot_yearly_sum,biomass_eff_crop_tot_yearly_tot)
   IF (printlev>=2) WRITE (numout,*)'biomass_eff_crop_lit_yearly_tot in PgC',biomass_eff_crop_tot_yearly_tot*1e-12
   ! Global grass
   biomass_eff_grass_yearly_sum = &
   & SUM(SUM(biomass_eff_grass(:,:), dim=2)*area(:)*contfrac(:))

   CALL allreduce_sum(biomass_eff_grass_yearly_sum,biomass_eff_grass_yearly_tot)
   IF (printlev>=2) WRITE (numout,*)'biomass_eff_grass_yearly_tot in PgC',biomass_eff_grass_yearly_tot*1e-12


   ! 5- Crop adjustement according to the global crop harvested supply

   sum_biomass_eff_crop_tot(:) = SUM(biomass_eff_crop_tot(:,:),dim=2)
   ! Global effective yearly harvest above biomass :
   biomass_eff_crop_tot_yearly_sum = &
   & SUM(sum_biomass_eff_crop_tot(:)*area(:)*contfrac(:))

   CALL allreduce_sum(biomass_eff_crop_tot_yearly_sum,biomass_eff_crop_tot_yearly_tot)

   IF(net_harvest_above_yearly_tot .GT. min_stomate) THEN
   hi_eff = biomass_eff_crop_tot_yearly_tot/ net_harvest_above_yearly_tot
   ELSE
   hi_eff = 0
   END IF
   IF (printlev>=2) WRITE (numout,*)'hi_eff:',hi_eff
   
   DO nlmi=1, nlm ! loop over livestock category 
     IF (hi_eff > un) THEN 
        biomass_eff_crop_tot(:,nlmi) = biomass_eff_crop_tot(:,nlmi)/hi_eff
     ELSE 
        biomass_eff_crop_tot(:,nlmi) = biomass_eff_crop_tot(:,nlmi)
     END IF

     biomass_eff_crop_alim(:,nlmi) = biomass_eff_crop_tot(:,nlmi)*(1-frac_lit(:,nlmi))
     biomass_eff_crop_lit(:,nlmi) = biomass_eff_crop_tot(:,nlmi)*frac_lit(:,nlmi)

     ! 6- Grass adjustment according to diet compostition only for ruminants (mix diet)
     IF (is_ruminant(nlmi))  THEN
       WHERE(diet(:,nlmi) .LT. (1-min_stomate))
         biomass_eff_grass(:,nlmi) = biomass_eff_crop_alim(:,nlmi)*(diet(:,nlmi)/(1-diet(:,nlmi)))
       END WHERE
     ELSE
       biomass_eff_grass(:,nlmi) = biomass_eff_grass(:,nlmi)
     END IF
   END DO

   ! Global effective yearly biomass :
   ! Global crop tot
   biomass_eff_crop_tot_yearly_sum = &
   & SUM(SUM(biomass_eff_crop_tot(:,:), dim=2)*area(:)*contfrac(:))

   CALL allreduce_sum(biomass_eff_crop_tot_yearly_sum,biomass_eff_crop_tot_yearly_tot)
   IF (printlev>=2) WRITE (numout,*)'biomass_eff_crop_tot_yearly_tot in PgC',biomass_eff_crop_tot_yearly_tot*1e-12
   ! Global litter 
   biomass_eff_crop_tot_yearly_sum = &
   & SUM(SUM(biomass_eff_crop_lit(:,:), dim=2)*area(:)*contfrac(:))

   CALL allreduce_sum(biomass_eff_crop_tot_yearly_sum,biomass_eff_crop_tot_yearly_tot)
   IF (printlev>=2) WRITE (numout,*)'biomass_eff_crop_lit_yearly_tot in PgC',biomass_eff_crop_tot_yearly_tot*1e-12
   ! Global grass
   biomass_eff_grass_yearly_sum = &
   & SUM(SUM(biomass_eff_grass(:,:), dim=2)*area(:)*contfrac(:))

   CALL allreduce_sum(biomass_eff_grass_yearly_sum,biomass_eff_grass_yearly_tot)
   IF (printlev>=2) WRITE (numout,*)'biomass_eff_grass_yearly_tot in PgC',biomass_eff_grass_yearly_tot*1e-12



  IF (printlev>=4) WRITE(numout,*) 'Leaving biomass_eff_estim'

  END SUBROUTINE

  ! ================================================================================================================================
  !!  SUBROUTINE   : diag_distr
  !!
  !>\BRIEF        Computes the estimation of the actual animal distribution
  !!             after adjustements of the resources
  !!
  !!  Subroutine called in the main routine
  !!
  !! MAIN OUTPUTS VARIABLE(S): distr_eff
  !! REFERENCE(S)   :
  !_ ================================================================================================================================

    SUBROUTINE diag_distr (npts, distr_eff, intake, biomass_eff_grass, biomass_eff_crop_alim)

      !! 0. Variable and parameter declaration

        !! 0.1 Input variables

        INTEGER(i_std), INTENT(in)                             :: npts             !! Domain size (unitless)
        REAL(r_std), DIMENSION(npts,nlm),INTENT(in)             :: intake           !! Intake required for each animal (kg C/year/ani)
        REAL(r_std),DIMENSION(npts,nlm),INTENT(in)              :: biomass_eff_grass    !! Actual grass biomass used by each animal (kg C/m2/yr)
        REAL(r_std),DIMENSION(npts,nlm),INTENT(in)              :: biomass_eff_crop_alim    !! Actual crop biomass used by each animal (kg C/m2/yr)

        !! 0.1 Modified variables
        REAL(r_std), DIMENSION(npts,nlm), INTENT(inout)      :: distr_eff           !! Livestock distribution map reajusted according to actual biomass grazed
                                                                                    !! (Heads/km2/yr)
        !! 0.4 Local variables

        INTEGER(i_std)                                        :: nlmi           !! Indices (unitless)
    !_ ================================================================================================================================

     ! DIAGNOSTIC : Calculation of the new animal densities from actual biomass grazed

   DO nlmi=1,nlm
     !distr_eff(:,nlmi) = (biomass_eff_grass(:,nlmi)*1e6)/(intake(:,nlmi)*diet(:,nlmi))
     distr_eff(:,nlmi) = (biomass_eff_grass(:,nlmi)+biomass_eff_crop_alim(:,nlmi))*1e6/intake(:,nlmi)
  END DO
    END SUBROUTINE



   ! ================================================================================================================================
   !!  SUBROUTINE   : manure_emis
   !!
   !>\BRIEF        Computes the nitrogen emissions from manure management
   !! - Compute ammonia (and other nitrogen species) emissions at each stage and by each animal
   !!  (unit : kg NH3-N/m2/yr)
   !! - Concerning field application and grazing manure, emissions are computed daily in Stomate.
   !!   And emssion factors are not used at the opposite of the other emission calcuations.
   !! - Compute the actual fraction of year where animals graze (xgraz_actual)
   !!
   !! Subroutine called in the main routine
   !!
   !! MAIN OUTPUTS VARIABLE(S): emis_house_tot, emis_yard, emis_sto_N,
   !! emis_sto_tot, m_tan_app, m_tan_graz
   !!
   !!
   !! REFERENCE(S)   :
   !!                 man : regional liquid and solid proportion in
   !! manure management (adapted from GLEAM 2011)
   !!
   !_ ================================================================================================================================


     SUBROUTINE manure_emis (npts, man, biomass_N, biomass_cropC_lit, &
       biomass_cropN_lit, emis_house_tot, emis_yard, emis_sto_N, &
       emis_sto_tot, m_app, m_graz_app)

   !! 0. Variable and parameter declaration

       !! 0.1 Input variables

       INTEGER(i_std), INTENT(in)                           :: npts             !! Domain size (unitless)
       REAL(r_std), DIMENSION(npts,nlm), INTENT(in)         :: man              !! Livestock manure proportion map per points
                                                                                !! handled as solid manure

       !! 0.2 Output variables
       REAL(r_std), DIMENSION(npts,nlm), INTENT(out)       :: emis_house_tot   !! Emissions from manure at housing (liquid + solid)
                                                                                !! for each animal (kg TAN/m2/yr)
       REAL(r_std), DIMENSION(npts,nlm), INTENT(out)        :: emis_yard         !! Emissions from manure at yard (liquid + solid)
                                                                                !! for each animal (kg TAN/m2/yr)
       REAL(r_std), DIMENSION(npts,nlm,nfrac_man), INTENT(out)        :: emis_sto_N       !! Total N Emissions from solid/liquid manure during storage
                                                                                !! for each animal (kg N/m2/yr)
       REAL(r_std), DIMENSION(npts,nlm), INTENT(out)        :: emis_sto_tot     !! NH3 Emissions from manure during storage (liquid + solid)
                                                                                !! for each animal (kg TAN/m2/yr)
       REAL(r_std), DIMENSION(npts,nlm,forms), INTENT(out)        :: m_app !! Liquid/solid manure produced for potential application
                                                                                !! for each animal (kg N/m2/yr)
       REAL(r_std), DIMENSION(npts,nlm,forms), INTENT(out)               :: m_graz_app       !! Manure deposited during grazing 
                                                                                  !! for each animal (kg TAN/m2/days)
       !! 0.3 Modified variables
       REAL(r_std), DIMENSION(npts,nlm), INTENT(inout)            :: biomass_N        !! Biomass ingested by each animal (kg N/m2/yr)
       REAL(r_std), DIMENSION(npts,nlm), INTENT(inout)            :: biomass_cropC_lit !! Biomass used as litter by each animal (kg C/m2/yr)
       REAL(r_std), DIMENSION(npts,nlm), INTENT(inout)            :: biomass_cropN_lit !! Biomass used as litter by each animal (kg N/m2/yr)

       !! 0.4 Local variables

       REAL(r_std), DIMENSION(npts,nlm)                      :: biomass_ex       !! Biomass excreted for each animal (kg N/m2/yr)
       REAL(r_std), DIMENSION(npts,nlm)                      :: m_graz           !! Biomass deposited during grazing
       REAL(r_std), DIMENSION(nlm)                           :: xgraz_actual     !! Actual time fraction passed at grazing
       REAL(r_std), DIMENSION(npts,nlm)                      :: m_house          !! Biomass deposited during housing
                                                                                 !! for each animal (kg N/m2/yr)
       REAL(r_std), DIMENSION(npts,nlm)                      :: m_yard           !! Biomass deposited at yard
                                                                                 !! for each animal (kg N/m2/yr)
      REAL(r_std), DIMENSION(npts,nlm)                      :: m_tan_yard        !! TAN deposited at yard
                                                                                 !! for each animal (kg TAN/m2/yr)
      REAL(r_std), DIMENSION(npts,nlm,nfrac_man)            :: m_tan_house       !! Solid/liquid TAN deposited at housing
                                                                                 !! for each animal (kg TAN/m2/yr)
      REAL(r_std), DIMENSION(npts,nlm,nfrac_man)            :: m_N_house       !! Solid/liquid total N deposited at housing
                                                                                 !! for each animal (kg N/m2/yr)
      REAL(r_std), DIMENSION(npts,nlm)                      :: m_N_bed         !! Solid total N for litter
                                                                                 !! for each animal (kg N/m2/yr)
      REAL(r_std), DIMENSION(npts,nlm)                      :: m_bed             !! Solid bedding
                                                                                 !! for each animal (kg /m2/yr)
      REAL(r_std), DIMENSION(npts,nlm,nfrac_man)            :: emis_house        !! Emissions from solid/liquid manure at housing
      REAL(r_std), DIMENSION(npts,nlm)                      :: m_tan_graz       !! TAN deposited during grazing
                                                                                      !! for each animal (kg TAN/m2/yr)
      REAL(r_std), DIMENSION(npts,nlm)                      :: m_tan_house_trans !! Solid TAN removed from house
                                                                                 !! for each animal (kg TAN/m2/yr)
      REAL(r_std), DIMENSION(npts,nlm)                      :: m_N_house_trans !! Total N removed from house
                                                                                 !! for each animal (kg N/m2/yr)
      REAL(r_std), DIMENSION(npts,nlm,nfrac_man)            :: m_tan_sto        !! Solid/liquid TAN that passes to storage
                                                                                 !! for each animal (kg TAN/m2/yr)
      REAL(r_std), DIMENSION(npts,nlm,nfrac_man)            :: m_N_sto         !! Solid/liquid total N that passes to storage
                                                                                 !! for each animal (kg TAN/m2/yr)
      REAL(r_std), DIMENSION(npts,nlm,nfrac_man)            :: m_N_app    !! Liquid/solid total N produced for potential application

      REAL(r_std), DIMENSION(npts,nlm,nfrac_man)            :: m_tan_app    !! Liquid/solid total TAN produced for potential application
                                                                           !! for each animal (kg N/m2/yr)
      REAL(r_std), DIMENSION(npts,nlm,nspecies,nfrac_man)   :: emis_sto         !! N specie emissions from solid/liquid manure during storage
                                                                                 !! for each animal (kg TAN/m2/yr)
     INTEGER(i_std)                                         :: nlmi,j,m,kk, ij   !! Indices (unitless)


   !_ ================================================================================================================================


   !! printlev is the level of diagnostic information, 0 (none) to 4 (full)
   IF (printlev>=2) WRITE(numout,*) 'Entering manure_emis'

   emis_house_tot = 0.0
   emis_yard = 0.0
   m_app = 0.0
   m_graz_app = 0.0
   emis_sto_tot = 0.0
   emis_sto_N = 0.0


   DO nlmi = 1, nlm ! Loop over animal category
     ! 1.1 Biomass ingested => biomass excreted
     biomass_ex(:,nlmi) = biomass_N(:,nlmi) * xexcreted(nlmi)
     ! 1.2 Manure deposited during :
     !                      - grazing (depend on the yard time)
     !                      - housing
     !                      - yard
     ! Perhaps make maps of parameters xyard, xgrazing ?
     xgraz_actual(nlmi) = xgrazing(nlmi) * (1 - xyard(nlmi))
     m_graz(:, nlmi) = xgraz_actual(nlmi) * biomass_ex(:,nlmi)
     m_house(:,nlmi) = (1-(xgraz_actual(nlmi) + xyard(nlmi))) * biomass_ex(:,nlmi)
     m_yard(:,nlmi) = xyard(nlmi) * biomass_ex(:,nlmi)
     ! 1.3 Proportion of excretal N that is TAN = N excreted in the urine
     m_tan_graz(:,nlmi) = xurine(nlmi) * m_graz(:,nlmi)
     m_tan_yard(:,nlmi) = xurine(nlmi) * m_yard(:,nlmi)
     m_graz_app(:,nlmi,itan) = m_tan_graz(:,nlmi)
     m_graz_app(:,nlmi,iorg) = m_graz(:,nlmi)-m_tan_graz(:,nlmi)
     
     DO m=1, nfrac_man
        m_tan_house(:,nlmi,m) = xurine(nlmi) * m_house(:,nlmi)
     END DO
     ! 1.4 Disssociation solid / liquid manures according to regional agricultural pratices
     ! 1.4.1 for TAN mass :
     m_tan_house(:,nlmi,isolid) = m_tan_house(:,nlmi,isolid) * man(:,nlmi)   ! solid fraction
     m_tan_house(:,nlmi,iliquid) = m_tan_house(:,nlmi,iliquid) * (1 - man(:,nlmi)) ! liquid fraction
     ! 1.4.2 for total N mass :
     m_N_house(:,nlmi,isolid) = m_house(:,nlmi) * man(:,nlmi)   ! solid fraction
     m_N_house(:,nlmi,iliquid) = m_house(:,nlmi) *(1 - man(:,nlmi))   ! liquid fraction
     ! 1.5. Bedding for litter calculated from solid part only
     m_bed(:,nlmi) = biomass_cropC_lit(:,nlmi) * man(:,nlmi) /0.32 ! => to get fresh matter from carbon
     m_N_bed(:,nlmi) = biomass_cropN_lit(:,nlmi) * man(:,nlmi)
     ! 2. Emission calculations : come from TAN masses only
     ! 2.1 Emissions in house
     DO m = 1, nfrac_man ! Loop over manure types (isolid, iliquid)
       emis_house(:,nlmi,m) = m_tan_house(:,nlmi,m) * ef_hou(nlmi,m)
     END DO

     emis_house_tot(:,nlmi) = SUM(emis_house(:,nlmi,:),dim=2) ! Addition of liquid/solid manures OUTPUT ?

     ! 2.2 Emissions in m_yard : no distinction between solid/liquid fractions
     emis_yard(:,nlmi) = m_tan_yard(:,nlmi) * ef_yard(nlmi)    ! OUTPUT ?
     ! 2.3 Amounts of total-N and TAN in solid manure that are removed
     !     from livestock housing :
     m_tan_house_trans(:,nlmi) = m_tan_house(:,nlmi,isolid) - (emis_house(:,nlmi,isolid) + &
                                (m_bed(:,nlmi)*ximm(nlmi)))

     WHERE(m_tan_house(:,nlmi,isolid) < (emis_house(:,nlmi,isolid) + &
                                (m_bed(:,nlmi)*ximm(nlmi))))
        m_tan_house_trans(:,nlmi) = 0.0
     ELSEWHERE
       m_tan_house_trans(:,nlmi)=m_tan_house_trans(:,nlmi)
     END WHERE
      m_N_house_trans(:,nlmi) = m_N_house(:,nlmi,isolid) + m_N_bed(:,nlmi) - &
                                  emis_house(:,nlmi,isolid)
      ! 2.4 Emissions during storage
      !     Amount of total -N and TAN that passes to the manure storage
      !     Hyp : all the amount is stored (no biogaz and direct application pathways)

      ! 2.4.1 for liquid fraction :
      m_tan_sto(:,nlmi,iliquid) = (m_tan_yard(:,nlmi) - emis_yard(:,nlmi)) + &
                             (m_tan_house(:,nlmi,iliquid) - emis_house(:,nlmi,iliquid))
      m_N_sto(:,nlmi,iliquid) = (m_yard(:,nlmi) - emis_house(:,nlmi,iliquid)) + &
                             (m_N_house(:,nlmi,iliquid) - emis_house(:,nlmi,iliquid))
      ! 2.4.2 for solid fraction :
      m_tan_sto(:,nlmi,isolid) = m_tan_house_trans(:,nlmi)
      m_N_sto(:,nlmi,isolid) = m_N_house_trans(:,nlmi)
      ! 2.4.3 Modified slurry mass containing the fraction of
      ! the organic N mineralised (fmin) to TAN before the gaseous emissions
      m_tan_sto(:,nlmi,iliquid) = m_tan_sto(:,nlmi,iliquid) + ((m_N_sto(:,nlmi,iliquid) - &
                               m_tan_sto(:,nlmi,iliquid)) * xmin(nlmi))

      ! 2.4.4 Emissions from solid and liquid TAN fraction :
      DO m = 1, nfrac_man ! Loop over manure types (isolid, iliquid)
        DO kk=1, nspecies ! Loop over chemical species
           emis_sto(:,nlmi,kk,m) = m_tan_sto(:,nlmi,m) * ef_sto(nlmi,kk,m)
        END DO
     ! Emissions from all the N species
        emis_sto_N(:,nlmi,m) = SUM(emis_sto(:,nlmi,:,m), dim=2) ! addition of all Nspecies !!!! conversion into N ??
      END DO

      ! Total ammonia emissions from storage
      emis_sto_tot(:,nlmi) = SUM(emis_sto(:,nlmi,inh3,:),dim=2) ! addition of solid and liquid manures
      ! 2.5 Emissions during application => monthly or daily ? Other subroutine
      ! 2.5.1 Manure produced yearly to apply daily ?
      DO m = 1, nfrac_man ! Loop over manure types (isolid, iliquid)
          m_tan_app(:,nlmi,m) = m_tan_sto(:,nlmi,m) - emis_sto_N(:,nlmi,m)
          m_N_app(:,nlmi,m) = m_N_sto(:,nlmi,m) - emis_sto_N(:,nlmi,m)
      END DO
      ! 2 output : org and tan to apply to soils
      m_app(:,nlmi,itan) = SUM(m_tan_app(:,nlmi,:),dim=2) ! addition of solid and liquid manures
      m_app(:,nlmi,iorg) = SUM(m_N_app(:,nlmi,:)-m_tan_app(:,nlmi,:),dim=2)

      ! 2.6 Emission during grazing no distinction between solid/liquid fractions
      ! emis_graz(:,nlmi) = m_tan_graz(:,nlmi) * ef_graz(nlmi) ! OUTPUT ?
   END DO

   IF (printlev>=4) WRITE(numout,*) 'Leaving manure_emis'

   END SUBROUTINE

  ! Back to soil ? which time step ?
  ! -> from grazing + application
  ! SUBROUTINE emis_nh3_main

  ! CALL manure_emis()

  ! DO nlmi = 1,nlm
  !   emis_tot(:,nlmi) = emis_house_tot(:,nlmi) + emis_yard(:,nlmi) + &
  !                      emis_sto_tot(:,nlmi)+ emis_appli_tot(:,nlmi) + &
  !                      emis_graz(:,nlmi)
  !   man_ret(:,nlmi) = man_applic_ret(:,nlmi) + m_graz_ret(:,nlmi)

  ! END DO

  ! END SUBROUTINE

 END MODULE emis_nh3
